#!/usr/bin/env python3

"""
This file is called template.py. It is a fully functional dummy experiment.
You can use this file as a start for programming your PsychoPy experiment.
The file template_stimuli.
The file template_stimuli.csv contains the following:
number;stimulus

0;"Graham Chapman"
1;"Eric Idle"
2;"Terry Gilliam"
3;"Terry Jones"
4;"John Cleese"
5;"Michael Palin"

"""
import os
import random
import csv
import pip
import platform
import psychopy
from psychopy import core, clock, visual, event, sound
from pylsl import StreamInfo, StreamOutlet
import my # import my own functions

# print info
print('python version: {}'.format(platform.python_version()))
print('psychopy version: {}'.format(psychopy.__version__))
print('Using {}(with {}) for sounds'.format(sound.audioLib, sound.audioDriver))

## Setup Section
win = visual.Window([800,600], fullscr=False, monitor="testMonitor", units='cm', color=(1.0, 0.968, 0.843))

# first create a new stream info (here we set the name to MyMarkerStream,
# the content-type to Markers, 1 channel, irregular sampling rate,
# and string-valued data) The last value would be the locally unique
# identifier for the stream as far as available, e.g.
# program-scriptname-subjectnumber (you could also omit it but interrupted
# connections wouldn't auto-recover). The important part is that the
# content-type is set to 'Markers', because then other programs will know how
#  to interpret the content
info = StreamInfo('MyMarkerStream', 'Markers', 1, 0, 'string', 'PsychopyMarkers43536')

# next make an outlet
outlet = StreamOutlet(info)
ppn = my.getString(win, "LSL started.")

# read stimuli file
trials = my.getStimulusInputFile('template_stimuli.csv')
numberColumn = 0     #number = trials[trialNumber][numberColumn]
nameColumn = 1       #name = trials[trialNumber][nameColumn]


# turn the text strings into stimuli
textStimuli = []
for trial in trials:
    # append this stimulus to the list of prepared stimuli
    textStimuli.append(visual.TextStim(win, text=trial[nameColumn], color = [-0.945,-0.831,-0.239])) 
    #imageStimuli.append(visual.ImageStim(win=window, size=[0.5,0.5], image="image/"+row[0]))
    #soundStimuli.append(visual.ImageStim(win=window, size=[0.5,0.5], image="image/"+row[0]))

#fixation cross
fixation = visual.ShapeStim(win, 
	vertices=((0, -0.5), (0, 0.5), (0,0), (-0.5,0), (0.5, 0)),
	lineWidth=3 ,
	closeShape=False,
	lineColor=[-0.945,-0.831,-0.239]
)

# open data output file
ppn = my.getString(win, "Please enter a participant number:")
outlet.push_sample([ppn])
datafile = my.openDataFile(ppn)
# connect it with a csv writer
writer = csv.writer(datafile, delimiter=";")
# create output file header
writer.writerow([
	"number", 
	"name", 
	"buttonPressed", 
	"fixationTime", 
	"textTime", 
	"responseTime"
	])
 
## Experiment Section
# show welcome screen
my.getCharacter(win, "Welcome, press any key to start")
startTime = clock.getTime() # clock is in seconds

for i, item in enumerate(trials):
	trial = trials[i]
	
	# present fixation
	fixation.draw()
	win.flip()
	fixationTime = clock.getTime()
	outlet.push_sample(['Fixation'])
	core.wait(0.980) # note how the real time will be very close to a multiple of the refresh time
	
	# present stimulus text and wait a maximum of 2 second for a response
	textStimuli[i].draw()
	win.flip()
	textTime = clock.getTime()
	outlet.push_sample([textStimuli[i].text])
	key = event.waitKeys(2.000)
	if key == None:
		outlet.push_sample(['NoKey'])
	else:
		outlet.push_sample(key)
	responseTime = clock.getTime()
	print("text: {}, key: {}, {}".format( trial[nameColumn], key, responseTime-textTime) )
	
	# black screen for 1000 ms
	win.flip()
	core.wait(1.000)
	
	# write result to data file
	if key==None:
		key=[]
		key.append("no key")
		
	writer.writerow([
		trial[numberColumn], 
		trial[nameColumn], 
		key[0],  
		"{:.3f}".format(fixationTime - startTime), 
		"{:.3f}".format(textTime - startTime), 
		"{:.3f}".format(responseTime - textTime),
		])
	
	if key[0]=='escape':
		break

# show goodbye screen
my.getCharacter(win, "Thank you for participating.")

## Closing Section
win.close()
core.quit()
