"""Example program to demonstrate how to read a multi-channel time-series
from LSL in a chunk-by-chunk manner (which is more efficient)."""

from pylsl import StreamInlet, resolve_stream, resolve_streams


def main():
    # first resolve an EEG stream on the lab network
    print("looking for an EEG stream...")
    streams = resolve_streams()
    for stream in streams:
        # do not set max_buflen because we *should not* be spammed by values
        inlet = StreamInlet(stream, max_buflen=1)
        info = inlet.info()
        name = info.name()
        print("-------------------")
        print("Stream name: " + name)
        # if target one stream, ignore false ones
        print("Nb channels: " + str(info.channel_count()))
        channelCount = info.channel_count()
        print("channelCount: " + str(channelCount))
        stream_freq = info.nominal_srate()
        print("Sampling frequency: " + str(stream_freq))
    streams = resolve_stream('name', 'BalanceBoard_stream')
    #streams = resolve_stream('name', 'BrainAmpSeries-Dev_1')

    # create a new inlet to read from the stream
    inlet = StreamInlet(streams[0])

    while True:
        # get a new sample (you can also omit the timestamp part if you're not
        # interested in it)
        chunk, timestamps = inlet.pull_chunk()
        if timestamps:
            #print(timestamps, chunk)
            print(len(chunk))


if __name__ == '__main__':
    main()
