"""Example program to demonstrate how to send a multi-channel time series to
LSL."""

from psychopy import prefs
#change the pref libraty to PTB and set the latency mode to high precision
# The audioLatencyMode can be set to an int from 1 to 4, where 1 denotes audio latency is not important, 
# 3 means high-precision mode. There is also the critical mode (4), which is basically the same as the mode 3, 
# with the difference that the script will break in case the system is not able to secure high precision.
prefs.hardware['audioLib'] = 'PTB'
prefs.hardware['audioLatencyMode'] = 3

import psychtoolbox as ptb
from psychopy import core, event, sound
from pylsl import StreamInfo, StreamOutlet, local_clock

import sys
import getopt

import time
from random import random as rand


def main(argv):
    srate = 100
    name = 'BioSemi'
    type = 'EEG'
    n_channels = 8
    help_string = 'SendData.py -s <sampling_rate> -n <stream_name> -t <stream_type>'
    try:
        opts, args = getopt.getopt(argv, "hs:c:n:t:", longopts=["srate=", "channels=", "name=", "type"])
    except getopt.GetoptError:
        print(help_string)
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print(help_string)
            sys.exit()
        elif opt in ("-s", "--srate"):
            srate = float(arg)
        elif opt in ("-c", "--channels"):
            n_channels = int(arg)
        elif opt in ("-n", "--name"):
            name = arg
        elif opt in ("-t", "--type"):
            type = arg

    # Set up LabStreamingLayer stream outlet
    info = StreamInfo(name='sound_stream', type='Markers', channel_count=1,
                      channel_format='int32', source_id='sound_stream')
    outlet = StreamOutlet(info)  # Broadcast the stream.
    beep = sound.Sound('beep_sound.wav')
    #Calculate the time stamp 500ms from now to allow enough time for the sound card to prepare the stimulus
    sample_stamp = ptb.GetSecs()+0.5;
    beep.play(when=sample_stamp)

    #push the trigger together with the timestamp through the LSL outlet
    markers = {'sound': [1]}
    outlet.push_sample(markers['sound'], sample_stamp)

    print("now sending data...")
    start_time = local_clock()
    sent_samples = 0
    while True:
        elapsed_time = local_clock() - start_time
        required_samples = int(srate * elapsed_time) - sent_samples
        for sample_ix in range(required_samples):
            # make a new random n_channels sample; this is converted into a
            # pylsl.vectorf (the data type that is expected by push_sample)
            mysample = [rand() for _ in range(n_channels)]
            # now send it
            outlet.push_sample(mysample)
            print(mysample)
        sent_samples += required_sampless
        # now send it and wait for a bit before trying again.
        time.sleep(0.01)


if __name__ == '__main__':
    main(sys.argv[1:])
