# This script redirects an SMI RED 500 iViewX eyegaze stream into an LSL stream.
# Written by Hubert Voogd, TSG, email: hubert.voogd@ru.nl

import socket
import threading
import win32api
import win32process
import ctypes
from pylsl import StreamInfo, StreamOutlet, local_clock

winmm = ctypes.WinDLL('winmm')
winmm.timeBeginPeriod(1)
win32process.SetPriorityClass(win32api.GetCurrentProcess(), win32process.HIGH_PRIORITY_CLASS)
 
msgFromClient = "Hello UDP Server....123"

bytesToSend = str.encode(msgFromClient)

# serverAddressPort = ("127.0.0.1", 4444)
# serverReadPort = ("127.0.0.1", 5555)
serverAddressPort = ("192.168.1.2", 4444)
serverReadPort = ("192.168.1.1", 5555)

bufferSize = 1024

 
# Create a UDP socket at client side
UDPSendSocket = socket.socket(family=socket.AF_INET, type=socket.SOCK_DGRAM)

UDPReadSocket = socket.socket(family=socket.AF_INET, type=socket.SOCK_DGRAM)
print("bind port: ", UDPReadSocket.bind(serverReadPort))

 
running = True


def ask_user():
    global running
    input("Press ENTER/RETURN to stop broadcasting iViewX gaze data into Lab Streaming Layer.")
    print('Closing thread.')
    running = False
    
    
thread_user = threading.Thread(target=ask_user).start()


'''
11.3.2.27 ET_INF
Returns information on the current used eye tracker.
Syntax Result
ET_INF ver version number: major.minor.build
ET_INF dev device name
Device name can be one of the following:
Hi-Speed
Hi-Speed Primate
HED 4 (USB)
RED
MRI-LR
MRI-SV
MRI-NNL
Custom
'''
UDPSendSocket.sendto(b'ET_INF dev' + b'\x0a\x0d\x00', serverAddressPort)
print("Asking device information")
msgFromServer = UDPReadSocket.recvfrom(bufferSize)
deviceName = (msgFromServer[0]).decode().rstrip('\x0a\x0d\x00')
deviceName = deviceName[8:len(deviceName)-1]
print('devicename: ', deviceName)
'''11.3.2.25 ET_FRM
Sets data format for data output. The syntax is similar to the 'C' string
formatting syntax. The format specification string is enclosed in quotation
marks. Each format specifier is marked by a preceding percentage (%)
symbol.
Parameters: format_string
Format specifiers:
- TS: timestamp in milliseconds (0 ...264/1000 ms)

- TU: timestamp in microseconds (0 ...264 μs)
- DX, DY: pupil diameter (0 ...232 pixels) x 32
- PX, PY: pupil position (± 231 pixels) x 32
- CX, CY: corneal reflex position (±231 pixels) x 32
- SX, SY: gaze position (± 231 pixels)
- SC: scene counter
- ET: eye type information, possible values:
l: left
r: right
b: binocular'''

# UDPClientSocket.sendto(b'ET_FRM %TS %SC %ET %SX %SY %DX %DY %PX %PY %CX %CY' + b'\x0a\x0d\x00', serverAddressPort)


# from https://github.com/labstreaminglayer/App-SMIEyetracker/blob/master/Legacy-SDK/mainwindow.cpp
print("sending stream format")
UDPSendSocket.sendto(b'ET_FRM \"%ET %SX %SY %DX %DY %PX %PY %CX %CY\"' + b' \x0a\x0d\x00', serverAddressPort)
print("stream format was sent")


# Send to server using created UDP socket
print("sending start stream command")
UDPSendSocket.sendto(b'ET_STR ' + b'\x0a\x0d\x00', serverAddressPort)
print("start stream command sent")
msgFromServer = UDPReadSocket.recvfrom(bufferSize)
#print("From server: ", msgFromServer)
msg2 = (msgFromServer[0]).decode().rstrip('\x0a\x0d\x00').split(' ')[1:]
freq = int(msg2[0])
print("Sample frequency: ", freq)
msgFromServer = UDPReadSocket.recvfrom(bufferSize)
#print("From server: ", msgFromServer)
msg = (msgFromServer[0]).decode().rstrip('\x0a\x0d\x00').split(' ')[1:]
UDPSendSocket.sendto(b'ET_EST ' + b'\x0a\x0d\x00', serverAddressPort)  # Stops the streaming of continuous data output.
# msgFromServer = UDPReadSocket.recvfrom(bufferSize)
if msg[0] == 'b':
    info = StreamInfo(name='iViewX ' + deviceName,
                      type='Gaze',
                      channel_count=16,
                      nominal_srate=freq,
                      channel_format='float32',
                      source_id=deviceName)
elif (msg[0] == 'l') or (msg[2] == 'r'):
    info = StreamInfo(name='iViewX ' + deviceName,
                      type='Gaze',
                      channel_count=8,
                      nominal_srate=freq,
                      channel_format='float32',
                      source_id=deviceName)
else:
    print("Unknown stream format: " + msg[2])
    info = StreamInfo(name='iViewX ' + deviceName,
                      type='Gaze',
                      channel_count=16,
                      nominal_srate=freq,
                      channel_format='float32',
                      source_id=deviceName)

print(info.name())

channel_count = 0
channels = info.desc().append_child("channels")

if (msg[0] == 'b') or (msg[0] == 'l'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Screen_X_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "ScreenX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Screen_Y_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "ScreenY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'r'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Screen_X_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "ScreenX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Screen_Y_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "ScreenY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'l'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Diameter_X_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "DiameterX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Diameter_Y_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "DiameterY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'r'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Diameter_X_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "DiameterX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Diameter_Y_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "DiameterY")
    c.append_child_value("unit", "pixels")

if (msg[0] == 'b') or (msg[0] == 'l'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Pupil_X_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "PupilX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Pupil_Y_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "PupilY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'r'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Pupil_X_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "PupilX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Pupil_Y_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "PupilY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'l'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Reflex_X_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "ReflexX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Reflex_Y_left")
    c.append_child_value("eye", "left")
    c.append_child_value("type", "ReflexY")
    c.append_child_value("unit", "pixels")
if (msg[0] == 'b') or (msg[0] == 'r'):
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Reflex_X_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "ReflexX")
    c.append_child_value("unit", "pixels")
    channel_count = channel_count + 1
    c = channels.append_child("channel")
    c.append_child_value("label", "Reflex_Y_right")
    c.append_child_value("eye", "right")
    c.append_child_value("type", "ReflexY")
    c.append_child_value("unit", "pixels")


c = info.desc().append_child("acquisition")
c.append_child_value("manufacturer", "SMI")
c.append_child_value("model", "iViewX")

# UDPReadSocket.setblocking(0)
outlet = StreamOutlet(info)
# Broadcast the stream.
UDPSendSocket.sendto(b'ET_STR ' + b'\x0a\x0d\x00', serverAddressPort)
msgFromServer = UDPReadSocket.recvfrom(bufferSize)
# print("From server: ", msgFromServer)
msg = (msgFromServer[0]).decode().rstrip('\x0a\x0d\x00').split(' ')[1:]
# print('Received: ', msg)
theTime = 0
print('Stream has started. LSL name: ', info.name())
print('Press ENTER/RETURN to stop broadcasting iViewX gaze data into Lab Streaming Layer.')
while running is True:
    lastTime = theTime
    msgFromServer = UDPReadSocket.recvfrom(bufferSize)
    theTime = local_clock()
    interval = theTime - lastTime
    # print('Interval: ', interval)
    msg = (msgFromServer[0]).decode().rstrip('\x0a\x0d\x00').split(' ')[1:]
    # print('MSG received: ', msg)
    if len(msg) > 1:
        ET = msg[0]
        # print('MSG[0] ==', msg[0])
        if msg[0] == 'b':
            SXl = float(msg[1])
            SXr = float(msg[2])
            SYl = float(msg[3])
            SYr = float(msg[4])
            DXl = float(msg[5])
            DXr = float(msg[6])
            DYl = float(msg[7])
            DYr = float(msg[8])
            PXl = float(msg[9])
            PXr = float(msg[10])
            PYl = float(msg[11])
            PYr = float(msg[12])
            CXl = float(msg[13])
            CXr = float(msg[14])
            CYl = float(msg[15])
            CYr = float(msg[16])
            # theChunk = [ET, SXl, SYl, SXr, SYr, DXl, DYl, DXr, DYr, PXl, PYl, PXr, PYr, CXl, CYl, CXr, CYr]
            theChunk = [SXl, SYl, SXr, SYr, DXl, DYl, DXr, DYr, PXl, PYl, PXr, PYr, CXl, CYl, CXr, CYr]
            # print('sent..', theChunk)
            # outlet.push_chunk(theChunk, timestamp=theTime)
            t1 = local_clock()
            outlet.push_sample(theChunk, timestamp=theTime)
            t2 = local_clock()
            # print('push_sample duration: ', t2 - t1)
        else:
            #print('Received: ', msg)
            #SXl = msg[1]
            #SYl = msg[2]
            #DXl = msg[3]
            #DYl = msg[4]
            #PXl = msg[5]
            #PYl = msg[6]
            #CXl = msg[7]
            #CYl = msg[8]
            #theChunk = [SXl, SYl, DXl, DYl, PXl, PYl, CXl, CYl]
            #print('sent..', theChunk)
            #outlet.push_chunk(theChunk, timestamp=theTime)
            pass

UDPSendSocket.sendto(b'ET_EST ' + b'\x0a\x0d\x00', serverAddressPort)  # Stops the streaming of continuous data output.
print('Stopping stream from iViewX.')
# msgFromServer = UDPReadSocket.recvfrom(bufferSize)
# print('Server responded: ', msgFromServer)
winmm.timeEndPeriod(1)


UDPReadSocket.close()
UDPSendSocket.close()

del outlet
del info
del channels
del UDPReadSocket
del UDPSendSocket
print('So long... and thanks for all the chunks.')
