#!/usr/bin/env python
# -*- coding: utf-8 -*-

import threading
from psychopy import visual, core, event
from pylsl import StreamInfo, StreamOutlet, local_clock
import time
import sys

# Create a window to draw in
win = visual.Window((800.0, 800.0), fullscr=False,allowGUI=True,waitBlanking=False)
sampling_freq_in = 1000.0

# Initialize some stimuli
fixSpot = visual.GratingStim(win, tex="none", mask="gauss",
    pos=(0, 0), size=(0.05, 0.05), color='black', autoLog=False)
myMouse = event.Mouse()  #  will use win by default

if sys.platform == "win32":
    from ctypes import *
    from ctypes.wintypes import DWORD
    class TIMECAPS(Structure):
            _fields_ = [("wPeriodMin",c_uint),
                                    ("wPeriodMax",c_uint)
                                    ]
    caps = TIMECAPS()
    windll.winmm.timeGetDevCaps(byref(caps), sizeof(caps))
    windll.winmm.timeBeginPeriod(caps.wPeriodMin) # set resolution in ms
    print("timed")

#setup streaming capture device
print ("Collecting data.....")

def getData():
    global mouse_dX, mouse_dY, keys
    mouse_dX, mouse_dY = myMouse.getPos()
    timeStamp = time.time() + (1.0/sampling_freq_in)
    while running:
        while timeStamp > time.time():
            pass
        #print(timeStamp - time.time())
        timeStamp = timeStamp + (1.0/sampling_freq_in)
        mouse_dX, mouse_dY = myMouse.getPos()
        outlet.push_sample([mouse_dX, mouse_dY])
        time.sleep(0.0005)
        
    print("exit getData")

info = StreamInfo(name='MyMouseStream', type='XY-Coord', channel_count=2, nominal_srate = sampling_freq_in, channel_format='float32', source_id='MouseStream')
outlet = StreamOutlet(info)  # Broadcast the stream. 

running = True
keys = event.getKeys()
threading.Thread(target=getData).start()
while not keys:
    keys = event.getKeys()
    #fixSpot.setPos([mouse_dX, mouse_dY])
    #fixSpot.draw()
    #win.flip()
    time.sleep(0.0001)

running = False
print("Stop")
