#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import threading
import time
import numpy as np
from pylsl import StreamInfo, StreamOutlet, local_clock
import cv2
import ctypes

#set sleep to 1ms accuracy
winmm = ctypes.WinDLL('winmm')
winmm.timeBeginPeriod(1)

#setup streaming capture device
def sendLSLFrames():
    stamp = local_clock()
    while running:
        time.sleep(0.001)
        while local_clock() < stamp:
            pass
        stamp = local_clock() + (1.0/freq)
        outlet.push_sample([frameCounter])#, local_clock())

def getWebcamData():
    global frameCounter
    global seconds
    global running
    global frame_width
    global frame_height

    print("opening webcam...")
    camera = cv2.VideoCapture(0)
    print("webcam open.")
    camera.set(cv2.CAP_PROP_FPS, frame_rate)
    camera.set(cv2.CAP_PROP_FRAME_WIDTH, frame_width)
    camera.set(cv2.CAP_PROP_FRAME_HEIGHT, frame_height)
    frame_width = int(camera.get(3))
    frame_height = int(camera.get(4))
    fps = camera.get(cv2.CAP_PROP_FPS)
    print("{},{},{}".format(frame_width, frame_height,fps))
    #fourcc = cv2.VideoWriter_fourcc('m','p','4','v')
    #video_writer = cv2.VideoWriter('output.m4v', fourcc, 30.0, (640, 360))#360p
    time_stamp = datetime.datetime.now().strftime('%Y-%m-%d %H-%M-%S')
    file_name = time_stamp +'_output.avi'
    fourcc = cv2.VideoWriter_fourcc('M','J','P','G')
    video_writer = cv2.VideoWriter(file_name, fourcc, frame_rate, (frame_width, frame_height))#360p
    
    grabbed, frame = camera.read()  # grab the current frame
    cv2.putText(frame, str(frameCounter), (20, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 0))
    frame = cv2.resize(frame, (int(frame_width/2),int(frame_height/2)))
    cv2.imshow("Frame", frame)
    print("WAIT for collecting webcam data, press any key.")
    while cv2.waitKey() == -1:
        pass
    print("START collecting webcam data, esc to stop.....")
    seconds = 0 

    while running:
        grabbed, frame = camera.read()  # grab the current frame +/-80ms delay inacurate
        if grabbed==True:
            frameCounter +=1
            #print(local_clock()-seconds)
            seconds = local_clock()
            time.sleep(0.001)
            cv2.putText(frame, str(frameCounter), (20, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 0))
            video_writer.write(frame)  # Write the video to the file system
            frame = cv2.resize(frame, (int(frame_width/2),int(frame_height/2)))
            cv2.imshow("Frame", frame)  # show the frame to our screen
        key = cv2.waitKey(1)
        if key == 27: # exit on ESC
            running = False

    camera.release()
    cv2.destroyAllWindows()

freq = 500
frame_rate = 60.0
frame_width = 1920
frame_height = 1080
#frame_width = 1280
#frame_height = 720
dataSize = 20
#info = StreamInfo(name='MyWebcamMarkerStream', type='Markers', channel_count=1, nominal_srate = 0, source_id='CharacterEvent')
info = StreamInfo(name='MyWebcamFrameStream', type='frameNR', channel_count=1, channel_format='int32', nominal_srate = freq, source_id='MyWebcamFrameStream')

outlet = StreamOutlet(info)  # Broadcast the stream. 

frameCounter = 1
running = True
threading.Thread(target=getWebcamData).start()
sendLSLFrames()
#send_data = np.zeros((1, dataSize),dtype=np.int32)
print("starting lsl")

while running == True:
    time.sleep(1)
    pass
    
print("Stop")
